/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#ifndef __ARBLKQUEUE_H__
#define __ARBLKQUEUE_H__

#include <pthread.h>

#define QUEUE_SIZE	128
#define QUEUE_WAIT	2048

enum	{ARBLKITEM_STATUS_NULL=0, ARBLKITEM_STATUS_TODO, ARBLKITEM_STATUS_PROGRESS, ARBLKITEM_STATUS_DONE};
enum	{ARBLKITEM_TYPE_NULL=0, ARBLKITEM_TYPE_BLOCK, ARBLKITEM_TYPE_DICO};

struct s_dico;

// if type==ARBLKITEM_TYPE_BLOCK, (blkdata,size,blkoffset) are used
// if type==ARBLKITEM_TYPE_DICO, (magic,dico) are used
struct s_arblkitem
{
	int 			type; // ARBLKITEM_TYPE_BLOCK or ARBLKITEM_TYPE_DICO
	s64			itemnum; // unique identifier of the item in the queue
	int			status; // compressed, being-compressed, not-yet-compressed
	struct s_arblkitem 	*next; // next block in the linked list
	// ------------------------------------------------------------------------------------
	char			*blkdata; // pointer to data block (compressed or uncompressed)
	u32			blksize; // block size
	u64			blkoffset; // where the block is located in the user file
	// ------------------------------------------------------------------------------------
	char			magic[FDARCHIVER_SIZEOF_MAGIC+1]; // magic for dico
	struct s_dico		*dico; // if (type==ARBLKITEM_TYPE_DICO)
};

struct s_arblkqueue
{
	struct s_arblkitem	*head;
	pthread_mutex_t		mutex;
	s64			curitemnum;
};

int arblkqueue_init(struct s_arblkqueue *l);
int arblkqueue_destroy(struct s_arblkqueue *l);
int arblkqueue_count(struct s_arblkqueue *l);
int arblkqueue_count_status(struct s_arblkqueue *l, int status);
int arblkqueue_print(struct s_arblkqueue *l);
s64 arblkqueue_get_first_block_todo(struct s_arblkqueue *q, char **blkdata, u64 *size, u64 *offset);
s64 arblkqueue_dequeue_first_if_done(struct s_arblkqueue *q, int *type, char **blkdata, u64 *size, u64 *offset, char *magic, struct s_dico **dico);
int arblkqueue_add_block(struct s_arblkqueue *q, char *blkdata, u64 blksize, u64 offset, int status);
int arblkqueue_add_block_wait(struct s_arblkqueue *q, char *blkdata, u64 blksize, u64 offset, int status);
int arblkqueue_replace_block(struct s_arblkqueue *q, s64 itemnum, char *newdata, u64 newsize, int newstatus);
int arblkqueue_add_dico(struct s_arblkqueue *q, struct s_dico *d, char *magic);
int arblkqueue_add_dico_wait(struct s_arblkqueue *q, struct s_dico *d, char *magic);
struct s_dico *arblkqueue_dequeue_header(struct s_arblkqueue *q, char *expectedmagic);
struct s_dico *arblkqueue_dequeue_header_wait(struct s_arblkqueue *q, char *expectedmagic);
int arblkqueue_dequeue_block(struct s_arblkqueue *q, char **blkdata, u64 *blksize, u64 *blkoffset);
int arblkqueue_dequeue_block_wait(struct s_arblkqueue *q, char **blkdata, u64 *blksize, u64 *blkoffset);
int arblkqueue_is_first_item_ready(struct s_arblkqueue *q);
int arblkqueue_check_next_item_wait(struct s_arblkqueue *q, int *type, char *magic);
int arblkqueue_check_next_item(struct s_arblkqueue *q, int *type, char *magic);
int arblkqueue_destroy_first_item(struct s_arblkqueue *q);

#endif // __ARBLKQUEUE_H__
