/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#include "types.h"
#include "fsarchiver.h"
#include "dico.h"
#include "common.h"
#include "archive.h"
#include "arblkqueue.h"
#include "comp_gzip.h"
#include "comp_bzip2.h"

int archive_init(struct s_archinfo *ai)
{
	if (!ai)
	{	dbgprintf(2, "ai is NULL\n");
		return -1;
	}

	memset(ai, 0, sizeof(struct s_archinfo));
	ai->archid=0;
	ai->locked=false;
	ai->terminate=false;
	arblkqueue_init(&ai->queue);
	return 0;
}

int archive_destroy(struct s_archinfo *ai)
{
	if (!ai)
	{	dbgprintf(2, "ai is NULL\n");
		return -1;
	}
	arblkqueue_destroy(&ai->queue);
	return 0;
}

int archive_create(struct s_archinfo *ai, char *archive)
{
	if (!ai)
	{	dbgprintf(2, "ai is NULL\n");
		return -1;
	}
	
	// generate an archive id
	ai->archid=generate_archive_id();
	
	ai->archfd=open(archive, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if (ai->archfd<0)
	{       sysprintf ("cannot create archive %s\n", archive);
		return -1;
	}
	
	if (lockf(ai->archfd, F_LOCK, 0)!=0)
	{	sysprintf("Cannot lock archive file: %s\n", archive);
		close(ai->archfd);
		return -1;
	}
	ai->locked=true;
	
	return 0;
}

int archive_open(struct s_archinfo *ai, char *archive)
{
	if (!ai)
	{	dbgprintf(2, "ai is NULL\n");
		return -1;
	}
	
	ai->archfd=open(archive, O_RDONLY);
	if (ai->archfd<0)
	{       sysprintf ("cannot open archive %s\n", archive);
		return -1;
	}

	return 0;
}

int archive_close(struct s_archinfo *ai)
{
	if (!ai)
	{	dbgprintf(2, "ai is NULL\n");
		return -1;
	}
		
	if (ai->locked)
		lockf(ai->archfd, F_ULOCK, 0);
	fsync(ai->archfd); // just in case the user reboots after it exits
	close(ai->archfd);

	return 0;
}
