/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#ifndef __FSARCHIVER_H__
#define __FSARCHIVER_H__

#include "types.h"

struct s_options
{
	u8	overwrite;
	u8	allowsaverw;
	int	verboselevel;
	int	debuglevel;
	int	compresslevel;
	int	compressjobs;
	char	destfs[128];
};

// global threads sync data
extern struct s_options g_options;
extern FILE *g_logfile;
extern int g_stop; // true if a thread exited with an error

// global threads sync functions
extern int get_terminate();

#define min(a,b) (((a)>(b)) ? (b):(a))
#define max(a,b) (((a)<(b)) ? (b):(a))
#define true 1
#define false 0

// used to print error messages (sysprintf when after a syscall, errprintf for normal function calls)
#define sysprintf(fmt, args...)					\
{ 	fprintf(stderr, "%s#%d: ", __FILE__ , __LINE__);	\
	myprintf(1, fmt, ## args);				\
	logprintf("%s#%d: ", __FILE__ , __LINE__);		\
	logprintf(fmt, ## args); 				\
}

#define errprintf(fmt, args...)					\
{	fprintf(stderr, "%s#%d: ", __FILE__ , __LINE__);	\
	myprintf(0, fmt, ## args);				\
	logprintf("%s#%d: ", __FILE__ , __LINE__);		\
	logprintf(fmt, ## args);				\
}

// normal messages: level=0 will always be printed, level=1 will be printed when options.verboselevel==1, ...
#define msgprintf(level, fmt, args...)				\
{	if (g_options.verboselevel>=level)			\
	{	myprintf(0, fmt, ## args);			\
	}							\
}

// messages to be printed to /var/log/fsarchiver.log when troubleshooting
#define dbgprintf(level, fmt, args...)				\
{	if (g_options.debuglevel>=level)			\
	{	logprintf("%s#%d: ", __FILE__ , __LINE__);	\
		logprintf(fmt, ## args);			\
	} 							\
}

#define FSARCHIVER_VERSION		"0.2.1"
#define FSARCHIVER_RELDATE		"2008-12-27"
#define FSARCHIVER_FILEFORMAT		"FsArCh_00K"

#define FDARCHIVER_MAXJOBS		32

#define FDARCHIVER_MAX_HEADER_SIZE	65536

#define FDARCHIVER_SIZEOF_MAGIC		4

#define FDARCHIVER_MAGIC_MAIN		"FsA0"
#define FDARCHIVER_MAGIC_FSYS		"FsYs"
#define FDARCHIVER_MAGIC_OBJT		"ObJt"
#define FDARCHIVER_MAGIC_XATTR		"XaTr"
#define FDARCHIVER_MAGIC_BLOCKHEADER	"BlKh"
#define FDARCHIVER_MAGIC_FILEFOOTER	"FoOt"

#define FDARCHIVER_SIZEOFKEY_INODES	sizeof(u64)
#define FDARCHIVER_SIZEOFKEY_DISKITEM	sizeof(u16)
#define FDARCHIVER_SIZEOFKEY_MAINHEAD	sizeof(u16)
#define FDARCHIVER_SIZEOFKEY_FSYSHEAD	sizeof(u16)
#define FDARCHIVER_SIZEOFKEY_XATTR	sizeof(u16)
#define FDARCHIVER_SIZEOFKEY_BLOCKHEAD	sizeof(u8)
#define FDARCHIVER_SIZEOFKEY_BLOCKFOOT	sizeof(u8)

#define DATA_MAX_BLOCK_SIZE		921600
#define DATA_DEF_BLOCK_SIZE		921600

enum 	{OBJTYPE_NULL=0, OBJTYPE_END, OBJTYPE_DIR, OBJTYPE_SYMLINK, OBJTYPE_REGFILE, OBJTYPE_HARDLINK, 
	OBJTYPE_CHARDEV, OBJTYPE_BLOCKDEV, OBJTYPE_FIFO, OBJTYPE_SOCKET};
enum	{COMPRESS_NULL=0, COMPRESS_NONE, COMPRESS_LZO, COMPRESS_GZIP, COMPRESS_BZIP2, COMPRESS_LZMA};
enum 	{DISKITEMKEY_NULL=0, DISKITEMKEY_PATH, DISKITEMKEY_OBJTYPE, DISKITEMKEY_SYMLINK, 
	DISKITEMKEY_HARDLINK, DISKITEMKEY_RDEV, DISKITEMKEY_MODE, DISKITEMKEY_SIZE, DISKITEMKEY_UID, 
	DISKITEMKEY_GID, DISKITEMKEY_ATIME, DISKITEMKEY_MTIME};
enum	{BLOCKHEADITEMKEY_NULL=0, BLOCKHEADITEMKEY_BLOCKSIZE, BLOCKHEADITEMKEY_BLOCKOFFSET, BLOCKHEADITEMKEY_COMPRESSALGO, 
	BLOCKHEADITEMKEY_FINALSIZE, BLOCKHEADITEMKEY_FINALCSUM};
enum	{BLOCKFOOTITEMKEY_NULL=0, BLOCKFOOTITEMKEY_MD5SUM};
enum	{MAINHEADKEY_NULL=0, MAINHEADKEY_FILEFORMATVER, MAINHEADKEY_PROGVERCREAT, MAINHEADKEY_ARCHIVEID, 
	MAINHEADKEY_CREATTIME, MAINHEADKEY_ARCHLABEL};
enum	{FSYSHEADKEY_NULL=0, FSYSHEADKEY_FILESYSTEM, FSYSHEADKEY_MNTPATH, FSYSHEADKEY_BYTESTOTAL, 
	FSYSHEADKEY_BYTESUSED, FSYSHEADKEY_FSLABEL, FSYSHEADKEY_FSUUID, FSYSHEADKEY_FSBLOCKSIZE, 
	FSYSHEADKEY_FSINODESIZE, FSYSHEADKEY_FSVERSION, FSYSHEADKEY_FSEXTDEFMNTOPT, FSYSHEADKEY_FSEXTFEATURES};

extern char *valid_magic[];

#endif // __FSARCHIVER_H__
