/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#ifndef __TYPES_H_
#define __TYPES_H_

#define get_cpu_var(p) (p)
#define __get_cpu_var(p) (p)
#define BITS_PER_LONG (sizeof(long) * 8)
#define ULONG_MAX       (~0UL)
#ifdef __CHECKER__
#define __force    __attribute__((force))
#define __bitwise__ __attribute__((bitwise))
#else
#define __force
#define __bitwise__
#endif

#ifndef __CHECKER__
#include <asm/types.h>
typedef __u32 u32;
typedef __s32 s32;
typedef __u64 u64;
typedef __s64 s64;
typedef __u16 u16;
typedef __s16 s16;
typedef __u8 u8;
typedef __s8 s8;
#else
typedef unsigned int u32;
typedef unsigned int __u32;
typedef unsigned long long u64;
typedef unsigned char u8;
typedef unsigned short u16;
typedef signed char s8;
#endif

#ifdef __CHECKER__
#define __CHECK_ENDIAN__
#define __bitwise __bitwise__
#else
#define __bitwise
#endif

typedef u16 __bitwise __le16;
typedef u16 __bitwise __be16;
typedef u32 __bitwise __le32;
typedef u32 __bitwise __be32;
typedef u64 __bitwise __le64;
typedef u64 __bitwise __be64;

// swapping macros
#define swab16(x) \
        ((u16)( \
                (((u16)(x) & (u16)0x00ffU) << 8) | \
                (((u16)(x) & (u16)0xff00U) >> 8) ))

#define swab32(x) \
        ((u32)( \
                (((u32)(x) & (u32)0x000000ffUL) << 24) | \
                (((u32)(x) & (u32)0x0000ff00UL) <<  8) | \
                (((u32)(x) & (u32)0x00ff0000UL) >>  8) | \
                (((u32)(x) & (u32)0xff000000UL) >> 24) ))

#define swab64(x) \
        ((u64)( \
                (u64)(((u64)(x) & (u64)0x00000000000000ffULL) << 56) | \
                (u64)(((u64)(x) & (u64)0x000000000000ff00ULL) << 40) | \
                (u64)(((u64)(x) & (u64)0x0000000000ff0000ULL) << 24) | \
                (u64)(((u64)(x) & (u64)0x00000000ff000000ULL) <<  8) | \
                (u64)(((u64)(x) & (u64)0x000000ff00000000ULL) >>  8) | \
                (u64)(((u64)(x) & (u64)0x0000ff0000000000ULL) >> 24) | \
                (u64)(((u64)(x) & (u64)0x00ff000000000000ULL) >> 40) | \
                (u64)(((u64)(x) & (u64)0xff00000000000000ULL) >> 56) ))

/* Macros to generate set/get funcs for the struct fields
 * assume there is a lefoo_to_cpu for every type, so lets make a simple
 * one for u8:
 */
#define le8_to_cpu(v) (v)
#define cpu_to_le8(v) (v)
#define __le8 u8

#if __BYTE_ORDER == __BIG_ENDIAN // CPU == big endian (sparc, ppc, ...)
#define cpu_to_le64(x) ((__force __le64)(u64)(swab64(x)))
#define le64_to_cpu(x) ((__force u64)(__le64)(swab64(x)))
#define cpu_to_le32(x) ((__force __le32)(u32)(swab32(x)))
#define le32_to_cpu(x) ((__force u32)(__le32)(swab32(x)))
#define cpu_to_le16(x) ((__force __le16)(u16)(swab16(x)))
#define le16_to_cpu(x) ((__force u16)(__le16)(swab16(x)))
#define be16_to_cpu(x) ((__force u16)(__be16)(x))
#define be32_to_cpu(x) ((__force u32)(__be32)(x))
#define be64_to_cpu(x) ((__force u64)(__be64)(x))
#else // CPU == little endian (intel)
#define cpu_to_le64(x) ((__force __le64)(u64)(x))
#define le64_to_cpu(x) ((__force u64)(__le64)(x))
#define cpu_to_le32(x) ((__force __le32)(u32)(x))
#define le32_to_cpu(x) ((__force u32)(__le32)(x))
#define cpu_to_le16(x) ((__force __le16)(u16)(x))
#define le16_to_cpu(x) ((__force u16)(__le16)(x))
#define be16_to_cpu(x) ((__force u16)(__be16)(swab16(x)))
#define be32_to_cpu(x) ((__force u32)(__be32)(swab32(x)))
#define be64_to_cpu(x) ((__force u64)(__be64)(swab64(x)))
#endif

#endif // __TYPES_H_
